# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.datetime_utils import serialize_datetime
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.hub_capacities_list_hub_capacities_response_body import HubCapacitiesListHubCapacitiesResponseBody
from ..types.hub_location_input_object_request_body import HubLocationInputObjectRequestBody
from ..types.hub_locations_create_hub_locations_response_body import HubLocationsCreateHubLocationsResponseBody
from ..types.hub_locations_list_hub_locations_response_body import HubLocationsListHubLocationsResponseBody
from ..types.hub_locations_update_hub_location_response_body import HubLocationsUpdateHubLocationResponseBody
from ..types.hub_skills_list_hub_skills_response_body import HubSkillsListHubSkillsResponseBody
from ..types.hubs_list_hubs_response_body import HubsListHubsResponseBody
from ..types.update_hub_location_request_body_request_body import UpdateHubLocationRequestBodyRequestBody

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawHubsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_hub_capacities(
        self,
        *,
        hub_id: str,
        capacity_ids: typing.Optional[str] = None,
        capacity_names: typing.Optional[str] = None,
        start_time: typing.Optional[dt.datetime] = None,
        end_time: typing.Optional[dt.datetime] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HubCapacitiesListHubCapacitiesResponseBody]:
        """
        Retrieve capacity types for a specific hub.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        hub_id : str
            The hub identifier

        capacity_ids : typing.Optional[str]
            Comma-separated list of capacity IDs for filtering.

        capacity_names : typing.Optional[str]
            Comma-separated list of capacity names for filtering.

        start_time : typing.Optional[dt.datetime]
            Time filter of when the capacity was updated, in RFC 3339 format

        end_time : typing.Optional[dt.datetime]
            Time filter of when the capacity was updated, in RFC 3339 format

        after : typing.Optional[str]
            If specified, should be the endCursor from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            Maximum number of objects to return. Default and maximum is 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HubCapacitiesListHubCapacitiesResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "hub/capacities",
            method="GET",
            params={
                "hubId": hub_id,
                "capacityIds": capacity_ids,
                "capacityNames": capacity_names,
                "startTime": serialize_datetime(start_time) if start_time is not None else None,
                "endTime": serialize_datetime(end_time) if end_time is not None else None,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubCapacitiesListHubCapacitiesResponseBody,
                    parse_obj_as(
                        type_=HubCapacitiesListHubCapacitiesResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_hub_location(
        self,
        id: str,
        *,
        data: UpdateHubLocationRequestBodyRequestBody,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HubLocationsUpdateHubLocationResponseBody]:
        """
        Update existing location by ID.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The unique Samsara ID of the hub location to update

        data : UpdateHubLocationRequestBodyRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HubLocationsUpdateHubLocationResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"hub/location/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=UpdateHubLocationRequestBodyRequestBody, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubLocationsUpdateHubLocationResponseBody,
                    parse_obj_as(
                        type_=HubLocationsUpdateHubLocationResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_hub_locations(
        self,
        *,
        hub_id: str,
        location_ids: typing.Optional[str] = None,
        customer_location_ids: typing.Optional[str] = None,
        start_time: typing.Optional[dt.datetime] = None,
        end_time: typing.Optional[dt.datetime] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HubLocationsListHubLocationsResponseBody]:
        """
        Retrieve locations for a specific hub.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        hub_id : str
            The hub identifier

        location_ids : typing.Optional[str]
            A comma-separated list of location IDs that can be used for filtering

        customer_location_ids : typing.Optional[str]
            A comma-separated list of customer provided location IDs that can be used for filtering

        start_time : typing.Optional[dt.datetime]
            Time filter of when the location was updated, in RFC 3339 format

        end_time : typing.Optional[dt.datetime]
            Time filter of when the location was updated, in RFC 3339 format

        after : typing.Optional[str]
            If specified, should be the endCursor from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            Maximum number of objects to return. Default and maximum is 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HubLocationsListHubLocationsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "hub/locations",
            method="GET",
            params={
                "hubId": hub_id,
                "locationIds": location_ids,
                "customerLocationIds": customer_location_ids,
                "startTime": serialize_datetime(start_time) if start_time is not None else None,
                "endTime": serialize_datetime(end_time) if end_time is not None else None,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubLocationsListHubLocationsResponseBody,
                    parse_obj_as(
                        type_=HubLocationsListHubLocationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_hub_locations(
        self,
        *,
        data: typing.Sequence[HubLocationInputObjectRequestBody],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HubLocationsCreateHubLocationsResponseBody]:
        """
        Create new locations.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        data : typing.Sequence[HubLocationInputObjectRequestBody]
            An array of location objects to be created or updated

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HubLocationsCreateHubLocationsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "hub/locations",
            method="POST",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=typing.Sequence[HubLocationInputObjectRequestBody], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubLocationsCreateHubLocationsResponseBody,
                    parse_obj_as(
                        type_=HubLocationsCreateHubLocationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_hub_skills(
        self,
        *,
        hub_id: str,
        skill_ids: typing.Optional[str] = None,
        skill_names: typing.Optional[str] = None,
        start_time: typing.Optional[dt.datetime] = None,
        end_time: typing.Optional[dt.datetime] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HubSkillsListHubSkillsResponseBody]:
        """
        Retrieve skills for a specific hub.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        hub_id : str
            The hub identifier

        skill_ids : typing.Optional[str]
            A comma-separated list of skill IDs that can be used for filtering.

        skill_names : typing.Optional[str]
            A comma-separated list of skill namess that can be used for filtering.

        start_time : typing.Optional[dt.datetime]
            Time filter of when the skill was updated, in RFC 3339 format

        end_time : typing.Optional[dt.datetime]
            Time filter of when the skill was updated, in RFC 3339 format

        after : typing.Optional[str]
            If specified, should be the endCursor from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            Maximum number of objects to return. Default and maximum is 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HubSkillsListHubSkillsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "hub/skills",
            method="GET",
            params={
                "hubId": hub_id,
                "skillIds": skill_ids,
                "skillNames": skill_names,
                "startTime": serialize_datetime(start_time) if start_time is not None else None,
                "endTime": serialize_datetime(end_time) if end_time is not None else None,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubSkillsListHubSkillsResponseBody,
                    parse_obj_as(
                        type_=HubSkillsListHubSkillsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_hubs(
        self,
        *,
        hub_ids: typing.Optional[str] = None,
        start_time: typing.Optional[dt.datetime] = None,
        end_time: typing.Optional[dt.datetime] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[HubsListHubsResponseBody]:
        """
        Retrieve all hubs (planners) in the organization. This endpoint supports pagination and filtering based on hub IDs and update times.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        hub_ids : typing.Optional[str]
            A comma-separated list of hub IDs for filtering results.

        start_time : typing.Optional[dt.datetime]
            Returns hubs updated after the specified time in UTC

        end_time : typing.Optional[dt.datetime]
            Returns hubs updated before the specified time in UTC

        after : typing.Optional[str]
            If specified, should be the endCursor from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            Maximum number of objects to return. Default and maximum is 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[HubsListHubsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "hubs",
            method="GET",
            params={
                "hubIds": hub_ids,
                "startTime": serialize_datetime(start_time) if start_time is not None else None,
                "endTime": serialize_datetime(end_time) if end_time is not None else None,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubsListHubsResponseBody,
                    parse_obj_as(
                        type_=HubsListHubsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawHubsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_hub_capacities(
        self,
        *,
        hub_id: str,
        capacity_ids: typing.Optional[str] = None,
        capacity_names: typing.Optional[str] = None,
        start_time: typing.Optional[dt.datetime] = None,
        end_time: typing.Optional[dt.datetime] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HubCapacitiesListHubCapacitiesResponseBody]:
        """
        Retrieve capacity types for a specific hub.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        hub_id : str
            The hub identifier

        capacity_ids : typing.Optional[str]
            Comma-separated list of capacity IDs for filtering.

        capacity_names : typing.Optional[str]
            Comma-separated list of capacity names for filtering.

        start_time : typing.Optional[dt.datetime]
            Time filter of when the capacity was updated, in RFC 3339 format

        end_time : typing.Optional[dt.datetime]
            Time filter of when the capacity was updated, in RFC 3339 format

        after : typing.Optional[str]
            If specified, should be the endCursor from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            Maximum number of objects to return. Default and maximum is 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HubCapacitiesListHubCapacitiesResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "hub/capacities",
            method="GET",
            params={
                "hubId": hub_id,
                "capacityIds": capacity_ids,
                "capacityNames": capacity_names,
                "startTime": serialize_datetime(start_time) if start_time is not None else None,
                "endTime": serialize_datetime(end_time) if end_time is not None else None,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubCapacitiesListHubCapacitiesResponseBody,
                    parse_obj_as(
                        type_=HubCapacitiesListHubCapacitiesResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_hub_location(
        self,
        id: str,
        *,
        data: UpdateHubLocationRequestBodyRequestBody,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HubLocationsUpdateHubLocationResponseBody]:
        """
        Update existing location by ID.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The unique Samsara ID of the hub location to update

        data : UpdateHubLocationRequestBodyRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HubLocationsUpdateHubLocationResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"hub/location/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=UpdateHubLocationRequestBodyRequestBody, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubLocationsUpdateHubLocationResponseBody,
                    parse_obj_as(
                        type_=HubLocationsUpdateHubLocationResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_hub_locations(
        self,
        *,
        hub_id: str,
        location_ids: typing.Optional[str] = None,
        customer_location_ids: typing.Optional[str] = None,
        start_time: typing.Optional[dt.datetime] = None,
        end_time: typing.Optional[dt.datetime] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HubLocationsListHubLocationsResponseBody]:
        """
        Retrieve locations for a specific hub.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        hub_id : str
            The hub identifier

        location_ids : typing.Optional[str]
            A comma-separated list of location IDs that can be used for filtering

        customer_location_ids : typing.Optional[str]
            A comma-separated list of customer provided location IDs that can be used for filtering

        start_time : typing.Optional[dt.datetime]
            Time filter of when the location was updated, in RFC 3339 format

        end_time : typing.Optional[dt.datetime]
            Time filter of when the location was updated, in RFC 3339 format

        after : typing.Optional[str]
            If specified, should be the endCursor from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            Maximum number of objects to return. Default and maximum is 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HubLocationsListHubLocationsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "hub/locations",
            method="GET",
            params={
                "hubId": hub_id,
                "locationIds": location_ids,
                "customerLocationIds": customer_location_ids,
                "startTime": serialize_datetime(start_time) if start_time is not None else None,
                "endTime": serialize_datetime(end_time) if end_time is not None else None,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubLocationsListHubLocationsResponseBody,
                    parse_obj_as(
                        type_=HubLocationsListHubLocationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_hub_locations(
        self,
        *,
        data: typing.Sequence[HubLocationInputObjectRequestBody],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HubLocationsCreateHubLocationsResponseBody]:
        """
        Create new locations.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        data : typing.Sequence[HubLocationInputObjectRequestBody]
            An array of location objects to be created or updated

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HubLocationsCreateHubLocationsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "hub/locations",
            method="POST",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=typing.Sequence[HubLocationInputObjectRequestBody], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubLocationsCreateHubLocationsResponseBody,
                    parse_obj_as(
                        type_=HubLocationsCreateHubLocationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_hub_skills(
        self,
        *,
        hub_id: str,
        skill_ids: typing.Optional[str] = None,
        skill_names: typing.Optional[str] = None,
        start_time: typing.Optional[dt.datetime] = None,
        end_time: typing.Optional[dt.datetime] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HubSkillsListHubSkillsResponseBody]:
        """
        Retrieve skills for a specific hub.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        hub_id : str
            The hub identifier

        skill_ids : typing.Optional[str]
            A comma-separated list of skill IDs that can be used for filtering.

        skill_names : typing.Optional[str]
            A comma-separated list of skill namess that can be used for filtering.

        start_time : typing.Optional[dt.datetime]
            Time filter of when the skill was updated, in RFC 3339 format

        end_time : typing.Optional[dt.datetime]
            Time filter of when the skill was updated, in RFC 3339 format

        after : typing.Optional[str]
            If specified, should be the endCursor from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            Maximum number of objects to return. Default and maximum is 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HubSkillsListHubSkillsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "hub/skills",
            method="GET",
            params={
                "hubId": hub_id,
                "skillIds": skill_ids,
                "skillNames": skill_names,
                "startTime": serialize_datetime(start_time) if start_time is not None else None,
                "endTime": serialize_datetime(end_time) if end_time is not None else None,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubSkillsListHubSkillsResponseBody,
                    parse_obj_as(
                        type_=HubSkillsListHubSkillsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_hubs(
        self,
        *,
        hub_ids: typing.Optional[str] = None,
        start_time: typing.Optional[dt.datetime] = None,
        end_time: typing.Optional[dt.datetime] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[HubsListHubsResponseBody]:
        """
        Retrieve all hubs (planners) in the organization. This endpoint supports pagination and filtering based on hub IDs and update times.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        hub_ids : typing.Optional[str]
            A comma-separated list of hub IDs for filtering results.

        start_time : typing.Optional[dt.datetime]
            Returns hubs updated after the specified time in UTC

        end_time : typing.Optional[dt.datetime]
            Returns hubs updated before the specified time in UTC

        after : typing.Optional[str]
            If specified, should be the endCursor from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            Maximum number of objects to return. Default and maximum is 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[HubsListHubsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "hubs",
            method="GET",
            params={
                "hubIds": hub_ids,
                "startTime": serialize_datetime(start_time) if start_time is not None else None,
                "endTime": serialize_datetime(end_time) if end_time is not None else None,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    HubsListHubsResponseBody,
                    parse_obj_as(
                        type_=HubsListHubsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

UsDriverRulesetOverrideCycle = typing.Union[
    typing.Literal[
        "USA Property (8/70)",
        "USA Property (7/60)",
        "USA Passenger (8/70)",
        "USA Passenger (7/60)",
        "Alaska Property (8/80)",
        "Alaska Property (7/70)",
        "Alaska Passenger (8/80)",
        "Alaska Passenger (7/70)",
        "California School/FLV (8/80)",
        "California Farm (8/112)",
        "California Property (8/80)",
        "California Flammable Liquid (8/80)",
        "California Passenger (8/80)",
        "California Motion Picture (8/80)",
        "Florida (8/80)",
        "Florida (7/70)",
        "Nebraska (8/80)",
        "Nebraska (7/70)",
        "North Carolina (8/80)",
        "North Carolina (7/70)",
        "Oklahoma (8/70)",
        "Oklahoma (7/60)",
        "Oregon (8/80)",
        "Oregon (7/70)",
        "South Carolina (8/80)",
        "South Carolina (7/70)",
        "Texas (7/70)",
        "Wisconsin (8/80)",
        "Wisconsin (7/70)",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .forms_table_column_object_response_body import FormsTableColumnObjectResponseBody
from .forms_table_row_object_response_body import FormsTableRowObjectResponseBody


class FormsTableValueObjectResponseBody(UniversalBaseModel):
    """
    The value of a table form input field.
    """

    columns: typing.List[FormsTableColumnObjectResponseBody] = pydantic.Field()
    """
    List of table columns.
    """

    rows: typing.List[FormsTableRowObjectResponseBody] = pydantic.Field()
    """
    List of table rows.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_stats_fault_codes_value_obdii_diagnostic_trouble_codes import (
    VehicleStatsFaultCodesValueObdiiDiagnosticTroubleCodes,
)


class VehicleStatsFaultCodesValueObdii(UniversalBaseModel):
    """
    Vehicle fault codes for passenger vehicles.
    """

    check_engine_light_is_on: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="checkEngineLightIsOn"),
        pydantic.Field(
            alias="checkEngineLightIsOn",
            description="True if the check engine light is illuminated (MIL status field is nonzero for any faults).",
        ),
    ] = None
    diagnostic_trouble_codes: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsFaultCodesValueObdiiDiagnosticTroubleCodes]],
        FieldMetadata(alias="diagnosticTroubleCodes"),
        pydantic.Field(alias="diagnosticTroubleCodes", description="Diagnostic trouble codes for passenger vehicles."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .work_order_attachment_object_response_body_processing_status import (
    WorkOrderAttachmentObjectResponseBodyProcessingStatus,
)


class WorkOrderAttachmentObjectResponseBody(UniversalBaseModel):
    """
    Work Order Attachment object.
    """

    id: str = pydantic.Field()
    """
    ID of the media record.
    """

    processing_status: typing_extensions.Annotated[
        WorkOrderAttachmentObjectResponseBodyProcessingStatus,
        FieldMetadata(alias="processingStatus"),
        pydantic.Field(
            alias="processingStatus",
            description="Status of the media record.  Valid values: `unknown`, `processing`, `finished`",
        ),
    ]
    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL containing a link to associated media content. Included if 'processingStatus' is 'finished'.
    """

    url_expires_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="urlExpiresAt"),
        pydantic.Field(
            alias="urlExpiresAt",
            description="Expiration time of the media record 'url'. UTC timestamp in RFC 3339 format.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BehaviorCountDetailsResponseBodyComparison = typing.Union[
    typing.Literal["EQUAL_TO", "GREATER_THAN", "GREATER_THAN_OR_EQUAL_TO", "LESS_THAN", "LESS_THAN_OR_EQUAL_TO"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .detection_log_safety_event_object_response_body_inbox_filter_reason import (
    DetectionLogSafetyEventObjectResponseBodyInboxFilterReason,
)


class DetectionLogSafetyEventObjectResponseBody(UniversalBaseModel):
    """
    Details on the associated safety event generated
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique Samsara ID (uuid) of the safety event. Only returned when safetyEvent.inboxEvent is true.
    """

    inbox_event: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="inboxEvent"),
        pydantic.Field(
            alias="inboxEvent",
            description="Whether a corresponding safety event was published to the Safety Inbox. Always returned.",
        ),
    ]
    inbox_filter_reason: typing_extensions.Annotated[
        typing.Optional[DetectionLogSafetyEventObjectResponseBodyInboxFilterReason],
        FieldMetadata(alias="inboxFilterReason"),
        pydantic.Field(
            alias="inboxFilterReason",
            description="The reason the detection was filtered out of the inbox. Only returned when safetyEvent.inboxEvent is false  Valid values: `overDailyLimit`, `overHourlyLimit`, `overTripLimit`, `belowConfidenceThreshold`, `belowSeverityThreshold`, `overEventRateLimit`, `geofenceFilter`, `belowNudgeThreshold`, `belowSpeedThreshold`, `nighttimeFilter`, `speedingFilter`, `inCabAlertOnly`, `unknown`",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

TireFaultCodeDetailsObjectRequestBodyManufacturer = typing.Union[
    typing.Literal[
        "MANUFACTURER_BENDIX",
        "MANUFACTURER_CONTINENTAL",
        "MANUFACTURER_DORAN",
        "MANUFACTURER_HENDRICKSON",
        "MANUFACTURER_INVALID",
        "MANUFACTURER_PRESSURE_PRO",
        "MANUFACTURER_UNIVERSAL_J1939",
        "MANUFACTURER_UNIVERSAL_R141",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .anticipation_brake_event_count import AnticipationBrakeEventCount
from .coasting_duration_ms import CoastingDurationMs
from .cruise_control_duration_ms import CruiseControlDurationMs
from .distance_driven_meters import DistanceDrivenMeters
from .drive_time_duration_ms import DriveTimeDurationMs
from .extended_driver_tiny_response import ExtendedDriverTinyResponse
from .fuel_consumed_ml import FuelConsumedMl
from .green_band_driving_duration_ms import GreenBandDrivingDurationMs
from .high_torque_ms import HighTorqueMs
from .idle_time_duration_ms import IdleTimeDurationMs
from .over_speed_ms import OverSpeedMs
from .power_take_off_duration_ms import PowerTakeOffDurationMs
from .total_brake_event_count import TotalBrakeEventCount
from .vehicle_summary import VehicleSummary


class DriverEfficiency(UniversalBaseModel):
    """
    Summary of a driver's efficiency.
    """

    anticipation_brake_event_count: typing_extensions.Annotated[
        typing.Optional[AnticipationBrakeEventCount],
        FieldMetadata(alias="anticipationBrakeEventCount"),
        pydantic.Field(alias="anticipationBrakeEventCount"),
    ] = None
    coasting_duration_ms: typing_extensions.Annotated[
        typing.Optional[CoastingDurationMs],
        FieldMetadata(alias="coastingDurationMs"),
        pydantic.Field(alias="coastingDurationMs"),
    ] = None
    cruise_control_duration_ms: typing_extensions.Annotated[
        typing.Optional[CruiseControlDurationMs],
        FieldMetadata(alias="cruiseControlDurationMs"),
        pydantic.Field(alias="cruiseControlDurationMs"),
    ] = None
    driver: typing.Optional[ExtendedDriverTinyResponse] = None
    green_band_driving_duration_ms: typing_extensions.Annotated[
        typing.Optional[GreenBandDrivingDurationMs],
        FieldMetadata(alias="greenBandDrivingDurationMs"),
        pydantic.Field(alias="greenBandDrivingDurationMs"),
    ] = None
    high_torque_ms: typing_extensions.Annotated[
        typing.Optional[HighTorqueMs], FieldMetadata(alias="highTorqueMs"), pydantic.Field(alias="highTorqueMs")
    ] = None
    over_speed_ms: typing_extensions.Annotated[
        typing.Optional[OverSpeedMs], FieldMetadata(alias="overSpeedMs"), pydantic.Field(alias="overSpeedMs")
    ] = None
    total_brake_event_count: typing_extensions.Annotated[
        typing.Optional[TotalBrakeEventCount],
        FieldMetadata(alias="totalBrakeEventCount"),
        pydantic.Field(alias="totalBrakeEventCount"),
    ] = None
    total_distance_driven_meters: typing_extensions.Annotated[
        typing.Optional[DistanceDrivenMeters],
        FieldMetadata(alias="totalDistanceDrivenMeters"),
        pydantic.Field(alias="totalDistanceDrivenMeters"),
    ] = None
    total_drive_time_duration_ms: typing_extensions.Annotated[
        typing.Optional[DriveTimeDurationMs],
        FieldMetadata(alias="totalDriveTimeDurationMs"),
        pydantic.Field(alias="totalDriveTimeDurationMs"),
    ] = None
    total_fuel_consumed_ml: typing_extensions.Annotated[
        typing.Optional[FuelConsumedMl],
        FieldMetadata(alias="totalFuelConsumedMl"),
        pydantic.Field(alias="totalFuelConsumedMl"),
    ] = None
    total_idle_time_duration_ms: typing_extensions.Annotated[
        typing.Optional[IdleTimeDurationMs],
        FieldMetadata(alias="totalIdleTimeDurationMs"),
        pydantic.Field(alias="totalIdleTimeDurationMs"),
    ] = None
    total_power_take_off_duration_ms: typing_extensions.Annotated[
        typing.Optional[PowerTakeOffDurationMs],
        FieldMetadata(alias="totalPowerTakeOffDurationMs"),
        pydantic.Field(alias="totalPowerTakeOffDurationMs"),
    ] = None
    vehicle_summaries: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleSummary]],
        FieldMetadata(alias="vehicleSummaries"),
        pydantic.Field(
            alias="vehicleSummaries",
            description="Summaries of vehicle efficiency for each vehicle the driver was driving during the given time period.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .minimal_route_stop_audit_logs_response_body import MinimalRouteStopAuditLogsResponseBody


class MinimalRouteAuditLogsResponseBody(UniversalBaseModel):
    """
    A single route. Only the fields that have changed are present in the response. All other fields, including the route id, will not be present in the response. For now, only routeStops are included since only Route Tracking updates are supported.
    """

    stops: typing.Optional[typing.List[MinimalRouteStopAuditLogsResponseBody]] = pydantic.Field(default=None)
    """
    The route stops in the route. Only stops that have been updated will be included in the response.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .safety_score_behavior_object_response_body import SafetyScoreBehaviorObjectResponseBody
from .safety_score_speeding_object_response_body import SafetyScoreSpeedingObjectResponseBody


class VehicleSafetyScoreResponseBody(UniversalBaseModel):
    """
    Safety score details for vehicle.
    """

    behaviors: typing.List[SafetyScoreBehaviorObjectResponseBody] = pydantic.Field()
    """
    Aggregated list of behaviors for the vehicle, one row for each unique type of behavior.
    """

    drive_distance_meters: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="driveDistanceMeters"),
        pydantic.Field(alias="driveDistanceMeters", description="Total sum of distance driven with the vehicle."),
    ]
    drive_time_milliseconds: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="driveTimeMilliseconds"),
        pydantic.Field(alias="driveTimeMilliseconds", description="Total time spent driving with the vehicle."),
    ]
    speeding: typing.List[SafetyScoreSpeedingObjectResponseBody] = pydantic.Field()
    """
    Aggregated list of speeding events for the vehicle, one row for each unique type of speeding.
    """

    vehicle_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="vehicleId"), pydantic.Field(alias="vehicleId", description="ID of the vehicle.")
    ]
    vehicle_score: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="vehicleScore"),
        pydantic.Field(
            alias="vehicleScore",
            description="Safety score for the vehicle. The score is a rounded number between 0-100.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FormSubmissionsPatchFormSubmissionRequestBodyStatus = typing.Union[
    typing.Literal["notStarted", "archived", "inProgress", "changesRequested", "approved"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

RouteSettingsRequestBodySequencingMethod = typing.Union[
    typing.Literal["unknown", "scheduledArrivalTime", "manual"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_polymorphic_user_object_response_body import FormsPolymorphicUserObjectResponseBody
from .qualification_field_input_object_response_body import QualificationFieldInputObjectResponseBody
from .qualification_owner_object_response_body import QualificationOwnerObjectResponseBody
from .qualification_record_response_object_response_body_record_status import (
    QualificationRecordResponseObjectResponseBodyRecordStatus,
)
from .qualification_type_reference_object_response_body import QualificationTypeReferenceObjectResponseBody


class QualificationRecordResponseObjectResponseBody(UniversalBaseModel):
    """
    Qualification Record response object.
    """

    created_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="Creation time. UTC timestamp in RFC 3339 format."),
    ]
    created_by: typing_extensions.Annotated[
        FormsPolymorphicUserObjectResponseBody, FieldMetadata(alias="createdBy"), pydantic.Field(alias="createdBy")
    ]
    expiration_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="expirationDate"),
        pydantic.Field(alias="expirationDate", description="Expiration date. UTC timestamp in RFC 3339 format."),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    fields: typing.List[QualificationFieldInputObjectResponseBody] = pydantic.Field()
    """
    List of field inputs in a qualification record.
    """

    id: str = pydantic.Field()
    """
    ID of the qualification record.
    """

    issue_date: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="issueDate"),
        pydantic.Field(alias="issueDate", description="Issue/effective date. UTC timestamp in RFC 3339 format."),
    ]
    owner: QualificationOwnerObjectResponseBody
    qualification_type: typing_extensions.Annotated[
        QualificationTypeReferenceObjectResponseBody,
        FieldMetadata(alias="qualificationType"),
        pydantic.Field(alias="qualificationType"),
    ]
    record_status: typing_extensions.Annotated[
        QualificationRecordResponseObjectResponseBodyRecordStatus,
        FieldMetadata(alias="recordStatus"),
        pydantic.Field(
            alias="recordStatus",
            description="Status of the qualification record.  Valid values: `active`, `archived`, `deleted`",
        ),
    ]
    updated_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(alias="updatedAtTime", description="Last updated time. UTC timestamp in RFC 3339 format."),
    ]
    updated_by: typing_extensions.Annotated[
        FormsPolymorphicUserObjectResponseBody, FieldMetadata(alias="updatedBy"), pydantic.Field(alias="updatedBy")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DriverRulesetObjectResponseBodyRestart = typing.Union[
    typing.Literal["34-hour Restart", "24-hour Restart", "36-hour Restart", "72-hour Restart"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.address import Address
from ..types.address_response import AddressResponse
from ..types.create_address_request_geofence import CreateAddressRequestGeofence
from ..types.list_addresses_response import ListAddressesResponse
from ..types.update_address_request_geofence import UpdateAddressRequestGeofence
from .raw_client import AsyncRawAddressesClient, RawAddressesClient
from .types.create_address_request_address_types_item import CreateAddressRequestAddressTypesItem
from .types.update_address_request_address_types_item import UpdateAddressRequestAddressTypesItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AddressesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAddressesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAddressesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAddressesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        created_after_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Address, ListAddressesResponse]:
        """
        Returns a list of all addresses in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        created_after_time : typing.Optional[str]
            A filter on data to have a created at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Address, ListAddressesResponse]
            List of all addresses in the organization

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        response = client.addresses.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            limit=limit,
            after=after,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            created_after_time=created_after_time,
            request_options=request_options,
        )

    def create(
        self,
        *,
        formatted_address: str,
        geofence: CreateAddressRequestGeofence,
        name: str,
        address_types: typing.Optional[typing.Sequence[CreateAddressRequestAddressTypesItem]] = OMIT,
        contact_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        latitude: typing.Optional[float] = OMIT,
        longitude: typing.Optional[float] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddressResponse:
        """
        Creates a new address in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        formatted_address : str
            The full street address for this address/geofence, as it might be recognized by Google Maps.

        geofence : CreateAddressRequestGeofence

        name : str
            Name of the address.

        address_types : typing.Optional[typing.Sequence[CreateAddressRequestAddressTypesItem]]
            Reporting location type associated with the address (used for ELD reporting purposes). Valid values: `yard`, `shortHaul`, `workforceSite`, `riskZone`, `industrialSite`, `alertsOnly`, `agricultureSource`, `avoidanceZone`, `knownGPSJammingZone`, `authorizedZone`, `unauthorizedZone`, `vendor`, `inventory`.

        contact_ids : typing.Optional[typing.Sequence[str]]
            An array of Contact IDs associated with this Address.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        latitude : typing.Optional[float]
            Latitude of the address. Will be geocoded from `formattedAddress` if not provided.

        longitude : typing.Optional[float]
            Longitude of the address. Will be geocoded from `formattedAddress` if not provided.

        notes : typing.Optional[str]
            Notes about the address.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this address.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddressResponse
            Newly created address object with ID.

        Examples
        --------
        from samsara import CreateAddressRequestGeofence, Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.addresses.create(
            formatted_address="350 Rhode Island St, San Francisco, CA",
            geofence=CreateAddressRequestGeofence(),
            name="Samsara HQ",
        )
        """
        _response = self._raw_client.create(
            formatted_address=formatted_address,
            geofence=geofence,
            name=name,
            address_types=address_types,
            contact_ids=contact_ids,
            external_ids=external_ids,
            latitude=latitude,
            longitude=longitude,
            notes=notes,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> AddressResponse:
        """
        Returns a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddressResponse
            An Address.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.addresses.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.addresses.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        address_types: typing.Optional[typing.Sequence[UpdateAddressRequestAddressTypesItem]] = OMIT,
        contact_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        formatted_address: typing.Optional[str] = OMIT,
        geofence: typing.Optional[UpdateAddressRequestGeofence] = OMIT,
        latitude: typing.Optional[float] = OMIT,
        longitude: typing.Optional[float] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddressResponse:
        """
        Update a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        address_types : typing.Optional[typing.Sequence[UpdateAddressRequestAddressTypesItem]]
            Reporting location type associated with the address (used for ELD reporting purposes). Valid values: `yard`, `shortHaul`, `workforceSite`, `riskZone`, `industrialSite`, `alertsOnly`, `agricultureSource`, `avoidanceZone`, `knownGPSJammingZone`, `authorizedZone`, `unauthorizedZone`, `vendor`, `inventory`.

        contact_ids : typing.Optional[typing.Sequence[str]]
            An array of Contact IDs associated with this Address.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        formatted_address : typing.Optional[str]
            The full street address for this address/geofence, as it might be recognized by Google Maps.

        geofence : typing.Optional[UpdateAddressRequestGeofence]

        latitude : typing.Optional[float]
            Latitude of the address. Will be geocoded from `formattedAddress` if not provided.

        longitude : typing.Optional[float]
            Longitude of the address. Will be geocoded from `formattedAddress` if not provided.

        name : typing.Optional[str]
            Name of the address.

        notes : typing.Optional[str]
            Notes about the address.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this address.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddressResponse
            Updated address object with ID.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.addresses.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            address_types=address_types,
            contact_ids=contact_ids,
            external_ids=external_ids,
            formatted_address=formatted_address,
            geofence=geofence,
            latitude=latitude,
            longitude=longitude,
            name=name,
            notes=notes,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data


class AsyncAddressesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAddressesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAddressesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAddressesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        created_after_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Address, ListAddressesResponse]:
        """
        Returns a list of all addresses in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        created_after_time : typing.Optional[str]
            A filter on data to have a created at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Address, ListAddressesResponse]
            List of all addresses in the organization

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.addresses.list()
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            limit=limit,
            after=after,
            parent_tag_ids=parent_tag_ids,
            tag_ids=tag_ids,
            created_after_time=created_after_time,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        formatted_address: str,
        geofence: CreateAddressRequestGeofence,
        name: str,
        address_types: typing.Optional[typing.Sequence[CreateAddressRequestAddressTypesItem]] = OMIT,
        contact_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        latitude: typing.Optional[float] = OMIT,
        longitude: typing.Optional[float] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddressResponse:
        """
        Creates a new address in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        formatted_address : str
            The full street address for this address/geofence, as it might be recognized by Google Maps.

        geofence : CreateAddressRequestGeofence

        name : str
            Name of the address.

        address_types : typing.Optional[typing.Sequence[CreateAddressRequestAddressTypesItem]]
            Reporting location type associated with the address (used for ELD reporting purposes). Valid values: `yard`, `shortHaul`, `workforceSite`, `riskZone`, `industrialSite`, `alertsOnly`, `agricultureSource`, `avoidanceZone`, `knownGPSJammingZone`, `authorizedZone`, `unauthorizedZone`, `vendor`, `inventory`.

        contact_ids : typing.Optional[typing.Sequence[str]]
            An array of Contact IDs associated with this Address.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        latitude : typing.Optional[float]
            Latitude of the address. Will be geocoded from `formattedAddress` if not provided.

        longitude : typing.Optional[float]
            Longitude of the address. Will be geocoded from `formattedAddress` if not provided.

        notes : typing.Optional[str]
            Notes about the address.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this address.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddressResponse
            Newly created address object with ID.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara, CreateAddressRequestGeofence

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.addresses.create(
                formatted_address="350 Rhode Island St, San Francisco, CA",
                geofence=CreateAddressRequestGeofence(),
                name="Samsara HQ",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            formatted_address=formatted_address,
            geofence=geofence,
            name=name,
            address_types=address_types,
            contact_ids=contact_ids,
            external_ids=external_ids,
            latitude=latitude,
            longitude=longitude,
            notes=notes,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> AddressResponse:
        """
        Returns a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddressResponse
            An Address.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.addresses.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.addresses.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        address_types: typing.Optional[typing.Sequence[UpdateAddressRequestAddressTypesItem]] = OMIT,
        contact_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        formatted_address: typing.Optional[str] = OMIT,
        geofence: typing.Optional[UpdateAddressRequestGeofence] = OMIT,
        latitude: typing.Optional[float] = OMIT,
        longitude: typing.Optional[float] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddressResponse:
        """
        Update a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        address_types : typing.Optional[typing.Sequence[UpdateAddressRequestAddressTypesItem]]
            Reporting location type associated with the address (used for ELD reporting purposes). Valid values: `yard`, `shortHaul`, `workforceSite`, `riskZone`, `industrialSite`, `alertsOnly`, `agricultureSource`, `avoidanceZone`, `knownGPSJammingZone`, `authorizedZone`, `unauthorizedZone`, `vendor`, `inventory`.

        contact_ids : typing.Optional[typing.Sequence[str]]
            An array of Contact IDs associated with this Address.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        formatted_address : typing.Optional[str]
            The full street address for this address/geofence, as it might be recognized by Google Maps.

        geofence : typing.Optional[UpdateAddressRequestGeofence]

        latitude : typing.Optional[float]
            Latitude of the address. Will be geocoded from `formattedAddress` if not provided.

        longitude : typing.Optional[float]
            Longitude of the address. Will be geocoded from `formattedAddress` if not provided.

        name : typing.Optional[str]
            Name of the address.

        notes : typing.Optional[str]
            Notes about the address.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this address.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddressResponse
            Updated address object with ID.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.addresses.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            address_types=address_types,
            contact_ids=contact_ids,
            external_ids=external_ids,
            formatted_address=formatted_address,
            geofence=geofence,
            latitude=latitude,
            longitude=longitude,
            name=name,
            notes=notes,
            tag_ids=tag_ids,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

VehicleStatsEngineImmobilizerWithDecorationState = typing.Union[
    typing.Literal["ignition_disabled", "ignition_enabled"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_field_definition_object_response_body import FormsFieldDefinitionObjectResponseBody
from .forms_polymorphic_user_object_response_body import FormsPolymorphicUserObjectResponseBody
from .qualification_type_response_object_response_body_entity_type import (
    QualificationTypeResponseObjectResponseBodyEntityType,
)


class QualificationTypeResponseObjectResponseBody(UniversalBaseModel):
    """
    Qualification type response object.
    """

    created_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime",
            description="Creation time of the qualification type. UTC timestamp in RFC 3339 format.",
        ),
    ]
    created_by: typing_extensions.Annotated[
        FormsPolymorphicUserObjectResponseBody, FieldMetadata(alias="createdBy"), pydantic.Field(alias="createdBy")
    ]
    entity_type: typing_extensions.Annotated[
        QualificationTypeResponseObjectResponseBodyEntityType,
        FieldMetadata(alias="entityType"),
        pydantic.Field(alias="entityType", description="Owner entity type  Valid values: `worker`, `asset`"),
    ]
    fields: typing.List[FormsFieldDefinitionObjectResponseBody] = pydantic.Field()
    """
    List of fields in the qualification type.
    """

    id: str = pydantic.Field()
    """
    Unique identifier of the qualification type.
    """

    name: str = pydantic.Field()
    """
    Name of the qualification type.
    """

    revision_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="revisionId"),
        pydantic.Field(alias="revisionId", description="Unique identifier of the qualification type revision."),
    ]
    updated_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime",
            description="Update time of the qualification type. UTC timestamp in RFC 3339 format.",
        ),
    ]
    updated_by: typing_extensions.Annotated[
        FormsPolymorphicUserObjectResponseBody, FieldMetadata(alias="updatedBy"), pydantic.Field(alias="updatedBy")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .attribute_attribute_type import AttributeAttributeType
from .attribute_entity_type import AttributeEntityType
from .attribute_unit import AttributeUnit
from .attribute_value_tiny import AttributeValueTiny


class Attribute(UniversalBaseModel):
    attribute_type: typing_extensions.Annotated[
        typing.Optional[AttributeAttributeType],
        FieldMetadata(alias="attributeType"),
        pydantic.Field(
            alias="attributeType",
            description="Denotes the data type of the attribute's values. Valid values: `single-select`, `multi-select`, `text`, `freeform-multi-select`, `number`, `date`.",
        ),
    ] = None
    entity_type: typing_extensions.Annotated[
        typing.Optional[AttributeEntityType],
        FieldMetadata(alias="entityType"),
        pydantic.Field(
            alias="entityType",
            description="Denotes the type of entity, driver or asset. Valid values: `driver`, `asset`.",
        ),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The samsara id of the attribute object.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of attribute.
    """

    number_values: typing_extensions.Annotated[
        typing.Optional[typing.List[float]],
        FieldMetadata(alias="numberValues"),
        pydantic.Field(
            alias="numberValues",
            description="Number values that can be associated with this attribute. Note: this field is `null` for `text` and `freeform-multi-select` attribute types.`",
        ),
    ] = None
    string_values: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="stringValues"),
        pydantic.Field(
            alias="stringValues",
            description="String values that can be associated with this attribute. Note: this field is `null` for `text` and `freeform-multi-select` attribute types.`",
        ),
    ] = None
    unit: typing.Optional[AttributeUnit] = pydantic.Field(default=None)
    """
    Unit of the attribute (only for Number attributes).
    """

    values: typing.Optional[typing.List[AttributeValueTiny]] = pydantic.Field(default=None)
    """
    Representation of values that includes ids. Note: this field is `null` for `text` and `freeform-multi-select` attribute types.`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .alert_object_driver_response_body import AlertObjectDriverResponseBody
from .alert_object_vehicle_response_body import AlertObjectVehicleResponseBody
from .event_location_response_body import EventLocationResponseBody


class SuddenFuelLevelDropResponseBody(UniversalBaseModel):
    """
    Details specific to Sudden Fuel Level Drop.
    """

    change_end_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="changeEndTime"),
        pydantic.Field(alias="changeEndTime", description="The end time of the fuel level change in RFC 3339 format."),
    ] = None
    change_start_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="changeStartTime"),
        pydantic.Field(
            alias="changeStartTime", description="The start time of the fuel level change in RFC 3339 format."
        ),
    ] = None
    driver: typing.Optional[AlertObjectDriverResponseBody] = None
    fuel_level_after_millipercent: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="fuelLevelAfterMillipercent"),
        pydantic.Field(
            alias="fuelLevelAfterMillipercent",
            description="The fuel level after the sudden fuel level drop in millipercents.",
        ),
    ] = None
    fuel_level_before_millipercent: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="fuelLevelBeforeMillipercent"),
        pydantic.Field(
            alias="fuelLevelBeforeMillipercent",
            description="The fuel level before the sudden fuel level drop in millipercents.",
        ),
    ] = None
    location: typing.Optional[EventLocationResponseBody] = None
    vehicle: typing.Optional[AlertObjectVehicleResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FormTemplateReferenceObjectResponseBody(UniversalBaseModel):
    """
    Form template reference object.
    """

    id: str = pydantic.Field()
    """
    ID of the form template.
    """

    revision_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="revisionId"),
        pydantic.Field(alias="revisionId", description="ID of the form template revision."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FormTemplateSectionObjectResponseBody(UniversalBaseModel):
    """
    Form Template section object.
    """

    field_index_first_inclusive: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="fieldIndexFirstInclusive"),
        pydantic.Field(
            alias="fieldIndexFirstInclusive",
            description="The index of the first field from the fields array that is in this section. Index 0 represents the first field definition of the fields array.",
        ),
    ]
    field_index_last_inclusive: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="fieldIndexLastInclusive"),
        pydantic.Field(
            alias="fieldIndexLastInclusive",
            description="The index of the last field from the fields array that is in this section.",
        ),
    ]
    id: str = pydantic.Field()
    """
    Identifier of the section.
    """

    label: str = pydantic.Field()
    """
    Label of the section.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .trailer_stat_reefer_door_state_zone_3_type_response_body_value import (
    TrailerStatReeferDoorStateZone3TypeResponseBodyValue,
)


class TrailerStatReeferDoorStateZone3TypeResponseBody(UniversalBaseModel):
    """
    The door state of the reefer.
    """

    time: str = pydantic.Field()
    """
    UTC timestamp in RFC 3339 format.
    """

    value: TrailerStatReeferDoorStateZone3TypeResponseBodyValue = pydantic.Field()
    """
    The door state of zone 2 of the reefer.  Valid values: `open`, `closed`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_stats_fault_codes_value_obdii_monitor_status_catalyst import (
    VehicleStatsFaultCodesValueObdiiMonitorStatusCatalyst,
)
from .vehicle_stats_fault_codes_value_obdii_monitor_status_comprehensive import (
    VehicleStatsFaultCodesValueObdiiMonitorStatusComprehensive,
)
from .vehicle_stats_fault_codes_value_obdii_monitor_status_egr import VehicleStatsFaultCodesValueObdiiMonitorStatusEgr
from .vehicle_stats_fault_codes_value_obdii_monitor_status_evap_system import (
    VehicleStatsFaultCodesValueObdiiMonitorStatusEvapSystem,
)
from .vehicle_stats_fault_codes_value_obdii_monitor_status_fuel import VehicleStatsFaultCodesValueObdiiMonitorStatusFuel
from .vehicle_stats_fault_codes_value_obdii_monitor_status_heated_catalyst import (
    VehicleStatsFaultCodesValueObdiiMonitorStatusHeatedCatalyst,
)
from .vehicle_stats_fault_codes_value_obdii_monitor_status_heated_o_2_sensor import (
    VehicleStatsFaultCodesValueObdiiMonitorStatusHeatedO2Sensor,
)
from .vehicle_stats_fault_codes_value_obdii_monitor_status_iso_sae_reserved import (
    VehicleStatsFaultCodesValueObdiiMonitorStatusIsoSaeReserved,
)
from .vehicle_stats_fault_codes_value_obdii_monitor_status_misfire import (
    VehicleStatsFaultCodesValueObdiiMonitorStatusMisfire,
)
from .vehicle_stats_fault_codes_value_obdii_monitor_status_o_2_sensor import (
    VehicleStatsFaultCodesValueObdiiMonitorStatusO2Sensor,
)
from .vehicle_stats_fault_codes_value_obdii_monitor_status_secondary_air import (
    VehicleStatsFaultCodesValueObdiiMonitorStatusSecondaryAir,
)


class VehicleStatsFaultCodesValueObdiiMonitorStatus(UniversalBaseModel):
    """
    Readings from engine sensors
    """

    catalyst: typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusCatalyst] = pydantic.Field(default=None)
    """
    Enum of monitor status:
    -U: Unsupported
    -N: Not Complete
    -R: Complete
    """

    comprehensive: typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusComprehensive] = pydantic.Field(
        default=None
    )
    """
    Enum of monitor status:
    -U: Unsupported
    -N: Not Complete
    -R: Complete
    """

    egr: typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusEgr] = pydantic.Field(default=None)
    """
    Enum of monitor status:
    -U: Unsupported
    -N: Not Complete
    -R: Complete
    """

    evap_system: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusEvapSystem],
        FieldMetadata(alias="evapSystem"),
        pydantic.Field(
            alias="evapSystem", description="Enum of monitor status:\n-U: Unsupported\n-N: Not Complete\n-R: Complete"
        ),
    ] = None
    fuel: typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusFuel] = pydantic.Field(default=None)
    """
    Enum of monitor status:
    -U: Unsupported
    -N: Not Complete
    -R: Complete
    """

    heated_catalyst: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusHeatedCatalyst],
        FieldMetadata(alias="heatedCatalyst"),
        pydantic.Field(
            alias="heatedCatalyst",
            description="Enum of monitor status:\n-U: Unsupported\n-N: Not Complete\n-R: Complete",
        ),
    ] = None
    heated_o_2_sensor: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusHeatedO2Sensor],
        FieldMetadata(alias="heatedO2Sensor"),
        pydantic.Field(
            alias="heatedO2Sensor",
            description="Enum of monitor status:\n-U: Unsupported\n-N: Not Complete\n-R: Complete",
        ),
    ] = None
    iso_sae_reserved: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusIsoSaeReserved],
        FieldMetadata(alias="isoSaeReserved"),
        pydantic.Field(
            alias="isoSaeReserved",
            description="Enum of monitor status:\n-U: Unsupported\n-N: Not Complete\n-R: Complete",
        ),
    ] = None
    misfire: typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusMisfire] = pydantic.Field(default=None)
    """
    Enum of monitor status:
    -U: Unsupported
    -N: Not Complete
    -R: Complete
    """

    not_ready_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="notReadyCount"),
        pydantic.Field(alias="notReadyCount", description="Count of the number of sensors reporting N: Not Complete"),
    ] = None
    o_2_sensor: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusO2Sensor],
        FieldMetadata(alias="o2Sensor"),
        pydantic.Field(
            alias="o2Sensor", description="Enum of monitor status:\n-U: Unsupported\n-N: Not Complete\n-R: Complete"
        ),
    ] = None
    secondary_air: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueObdiiMonitorStatusSecondaryAir],
        FieldMetadata(alias="secondaryAir"),
        pydantic.Field(
            alias="secondaryAir", description="Enum of monitor status:\n-U: Unsupported\n-N: Not Complete\n-R: Complete"
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .field_types_object_response_body_field_type import FieldTypesObjectResponseBodyFieldType
from .multiple_choice_field_type_meta_data_object_response_body import MultipleChoiceFieldTypeMetaDataObjectResponseBody
from .number_field_type_meta_data_object_response_body import NumberFieldTypeMetaDataObjectResponseBody
from .signature_field_type_meta_data_object_response_body import SignatureFieldTypeMetaDataObjectResponseBody


class FieldTypesObjectResponseBody(UniversalBaseModel):
    field_type: typing_extensions.Annotated[
        FieldTypesObjectResponseBodyFieldType,
        FieldMetadata(alias="fieldType"),
        pydantic.Field(
            alias="fieldType",
            description="The type of value this field can have.  Valid values: `photo`, `string`, `number`, `multipleChoice`, `signature`, `dateTime`, `scannedDocument`, `barcode`",
        ),
    ]
    label: str = pydantic.Field()
    """
    The name of the field type.
    """

    multiple_choice_field_type_meta_data: typing_extensions.Annotated[
        typing.Optional[typing.List[MultipleChoiceFieldTypeMetaDataObjectResponseBody]],
        FieldMetadata(alias="multipleChoiceFieldTypeMetaData"),
        pydantic.Field(
            alias="multipleChoiceFieldTypeMetaData", description="A list of the multiple choice field option labels."
        ),
    ] = None
    number_field_type_meta_data: typing_extensions.Annotated[
        typing.Optional[NumberFieldTypeMetaDataObjectResponseBody],
        FieldMetadata(alias="numberFieldTypeMetaData"),
        pydantic.Field(alias="numberFieldTypeMetaData"),
    ] = None
    required_field: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="requiredField"),
        pydantic.Field(alias="requiredField", description="The indicator that states if the field is required."),
    ]
    signature_field_type_meta_data: typing_extensions.Annotated[
        typing.Optional[SignatureFieldTypeMetaDataObjectResponseBody],
        FieldMetadata(alias="signatureFieldTypeMetaData"),
        pydantic.Field(alias="signatureFieldTypeMetaData"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .user_auth_type import UserAuthType
from .user_email import UserEmail
from .user_id import UserId
from .user_name import UserName
from .user_roles import UserRoles


class User(UniversalBaseModel):
    """
    A user object.
    """

    auth_type: typing_extensions.Annotated[
        UserAuthType, FieldMetadata(alias="authType"), pydantic.Field(alias="authType")
    ]
    email: UserEmail
    id: UserId
    name: UserName
    roles: UserRoles

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ReportsDatasetFieldResponseBodyUnit = typing.Union[
    typing.Literal[
        "amperes",
        "bars",
        "cad",
        "chf",
        "days",
        "decimalDegrees",
        "degreesCelsius",
        "degreesFahrenheit",
        "eur",
        "feet",
        "gForces",
        "gallons",
        "gallonsPerHour",
        "gallonsPerKilogram",
        "gallonsPerMile",
        "gbp",
        "gramsPerLiter",
        "gramsPerMeter",
        "hertz",
        "hours",
        "imperialGallons",
        "imperialGallonsPerHour",
        "imperialGallonsPerKilogram",
        "imperialGallonsPerMile",
        "inches",
        "kelvins",
        "kilograms",
        "kilogramsGaseousFuel",
        "kilogramsPer100KilometerGaseousFuel",
        "kilogramsPerGallon",
        "kilogramsPerKilometer",
        "kilogramsPerLiter",
        "kilogramsPerMile",
        "kilometers",
        "kilometersPerHour",
        "kilometersPerLiter",
        "kilometersPerLiterGaseousFuel",
        "kilopascals",
        "kilowattHours",
        "liters",
        "litersGaseousFuel",
        "litersPer100Kilometers",
        "litersPer100KilometersGaseousFuel",
        "litersPerHour",
        "litersPerKilogram",
        "litersPerKilometer",
        "litersPerMeter",
        "litersPerTonne",
        "meters",
        "metersPerSecond",
        "miles",
        "milesPerGallon",
        "milesPerHour",
        "milesPerImperialGallon",
        "milesPerKilogramGaseousFuel",
        "milesPerUSGallonGaseousFuel",
        "milliknots",
        "milliseconds",
        "millivolts",
        "minutes",
        "months",
        "mxn",
        "percents",
        "pounds",
        "poundsPerGallon",
        "poundsPerLiter",
        "poundsPerMile",
        "poundsPerSquareInch",
        "revolutionsPerMinute",
        "seconds",
        "unknown",
        "usGallonsGaseousFuel",
        "usd",
        "voltAmperes",
        "voltAmperesReactive",
        "volts",
        "wattHours",
        "watts",
        "weeks",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .asset_response_body_regulation_mode import AssetResponseBodyRegulationMode
from .asset_response_body_type import AssetResponseBodyType
from .goa_tag_tiny_response_response_body import GoaTagTinyResponseResponseBody


class AssetResponseBody(UniversalBaseModel):
    """
    Representation of a vehicle trailer or other equipment to be tracked.
    """

    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="The time the asset was created in RFC 3339 format."),
    ]
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: str = pydantic.Field()
    """
    The unique ID of the asset.
    """

    license_plate: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="licensePlate"),
        pydantic.Field(alias="licensePlate", description="The license plate of the asset."),
    ] = None
    make: typing.Optional[str] = pydantic.Field(default=None)
    """
    The OEM/manufacturer of the asset. Updates to this field are restricted.
    """

    model: typing.Optional[str] = pydantic.Field(default=None)
    """
    The model of the asset. Updates to this field are restricted.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The human-readable name of the asset. This is set by a fleet administrator and will appear in both Samsara’s cloud dashboard as well as the Samsara Driver mobile app. By default, this name is the serial number of the Samsara Asset Gateway. It can be set or updated through the Samsara Dashboard or through the API at any time.
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    These are generic notes about the asset. Can be set or updated through the Samsara Dashboard or the API at any time.
    """

    readings_ingestion_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="readingsIngestionEnabled"),
        pydantic.Field(
            alias="readingsIngestionEnabled",
            description="Indicates whether the asset is expected to have data ingested using the Readings API.",
        ),
    ] = None
    regulation_mode: typing_extensions.Annotated[
        typing.Optional[AssetResponseBodyRegulationMode],
        FieldMetadata(alias="regulationMode"),
        pydantic.Field(
            alias="regulationMode",
            description="Whether or not the asset is regulated, unregulated (non-CMV), or a mixed use unregulated asset. Primarily used with vehicles.  Valid values: `mixed`, `regulated`, `unregulated`",
        ),
    ] = None
    serial_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="serialNumber"),
        pydantic.Field(
            alias="serialNumber",
            description="The serial number of the asset. This can be an internal serial number or used to hold legacy VIN/PIN numbers such as ones of shorter lengths.",
        ),
    ] = None
    tags: typing.Optional[typing.List[GoaTagTinyResponseResponseBody]] = pydantic.Field(default=None)
    """
    The list of [tags](https://kb.samsara.com/hc/en-us/articles/360026674631-Using-Tags-and-Tag-Nesting) associated with the Asset.
    """

    type: typing.Optional[AssetResponseBodyType] = pydantic.Field(default=None)
    """
    The operational context in which the asset interacts with the Samsara system. Examples: Vehicle (eg: truck, bus...), Trailer (eg: dry van, reefer, flatbed...), Powered Equipment (eg: dozer, crane...), Unpowered Equipment (eg: container, dumpster...), or Uncategorized.  Valid values: `uncategorized`, `trailer`, `equipment`, `unpowered`, `vehicle`
    """

    updated_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(alias="updatedAtTime", description="The time the asset was last updated in RFC 3339 format."),
    ]
    vin: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique 17-digit VIN (Vehicle Identification Number) or PIN (Product Identification Number) of the asset.
    """

    year: typing.Optional[int] = pydantic.Field(default=None)
    """
    The model year of the asset. Updates to this field are restricted.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SafetyEventBehaviorLabelsResponseBodyType = typing.Union[
    typing.Literal[
        "Acceleration",
        "Braking",
        "Crash",
        "DefensiveDriving",
        "DidNotYield",
        "Drinking",
        "Drowsy",
        "Eating",
        "EatingDrinking",
        "EdgeDistractedDriving",
        "EdgeRailroadCrossingViolation",
        "FollowingDistance",
        "FollowingDistanceModerate",
        "FollowingDistanceSevere",
        "ForwardCollisionWarning",
        "GenericDistraction",
        "GenericTailgating",
        "HarshTurn",
        "HighSpeedSuddenDisconnect",
        "Invalid",
        "LaneDeparture",
        "LateResponse",
        "MobileUsage",
        "NearCollison",
        "NoSeatbelt",
        "ObstructedCamera",
        "OperationalEvent",
        "Passenger",
        "PolicyViolationMask",
        "ProtectiveEquipment",
        "RanRedLight",
        "RearCollisionWarning",
        "RollingStop",
        "RolloverProtection",
        "Smoking",
        "Speeding",
        "VehicleInBlindSpotWarning",
        "VulnerableRoadUserCollisionWarning",
        "YawControl",
    ],
    typing.Any,
]
